/******************************************************************************
 * @file     main.c
 * @brief    Main program body
 * @version  V1.8.0
 * @date     31-Dec-2021
 *
 * @note
 * Copyright (C) 2021 Spintrol Limited Corporation. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/

#include"spc1168.h"
#include<stdio.h>

/* As descripted below, ADC result convert to voltage can calculate as the follow */
#define           ValueToVoltage(x)        ((x * 3657) / 8192) /1000

int16_t           i16VSP;


/*************************************************************************************************************************
 *
 * @brief      In this case, we use one GPIO as the input, then getting the GPIO voltage digital code from the result register.
 *
 *************************************************************************************************************************/


int main()
{
  FLASH_WALLOW();
  FLASH_SetTiming(200000000);
  /* Disable flash write access after flash operation had done */
  FLASH_WDIS();

  CLOCK_InitWithRCO(CLOCK_HCLK_200MHZ);

  Delay_Init();

  /*
   * Init the UART
   *
   * 1.Set the GPIO34/35 as UART FUNC
   *
   * 2.Enable the UART CLK
   *
   * 3.Set the rest para
   */
  GPIO_SetPinChannel(GPIO_34,GPIO34_UART_TXD);
  GPIO_SetPinChannel(GPIO_35,GPIO35_UART_RXD);
  CLOCK_EnableModule(UART_MODULE);
  UART_Init(UART,38400);

  /*ADC Init*/
  ADC_EasyInit1(ADC_SOC_0,ADC8_GPIO8,ADCTRIG_Software);

  while(1)
  {
    /* Use software to trigger ADC SOC0 to work */
    ADC_SoftwareTrigger(ADC_SOC_0);

    /* Wait until ADC conversion finished (Interrupt flag was set) */
    while(!ADC_GetIntFlag(ADC_SOC_0));

    /* Result gotten from 'ADC_GetResult()' can be a negtive value or a positive value */
    i16VSP = ADC_GetResult(ADC_SOC_0);
    printf("ADC SOC0 Voltage = %2fV(Raw data is %d)\n", (double)ValueToVoltage(i16VSP),i16VSP);

    /* Result gotten from 'ADC_GetTrimResult1()' is a positive value */
    i16VSP = ADC_GetTrimResult1(ADC_SOC_0);
    printf("ADC SOC0 Voltage = %2fV(Trim data is %d)\n \n", (double)ValueToVoltage(i16VSP),i16VSP);

    /* Clear ADC SOC0 INT flag */
    ADC_ClearInt(ADC_SOC_0);

    Delay_Ms(500);
  }
}
