/******************************************************************************
 * @file     main.c
 * @brief    Main program body
 * @version  V1.8.0
 * @date     31-Dec-2021
 *
 * @note
 * Copyright (C) 2021 Spintrol Limited Corporation. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/

#include"spc1168.h"
#include<stdio.h>

#define                 OffsetADDRInFlash         0x10000                             /*Offset in entire flash memory*/
#define                 FlashBaseADDR             0x10000000                          /*Flash ADDR in SYS memory layout*/
#define                 OperationADDR             FlashBaseADDR + OffsetADDRInFlash
#define                 OperationLEN              0x400                               /*1kB data*/
#define                 WP_START_ADDR             0x10010000
#define                 WP_END_ADDR               0x10010400

ErrorStatus             status;
uint32_t                pu32Buf[OperationLEN] = {0};
uint32_t                i;

void Flash_Operation(void)
{
  /* Erase flash sector */
  FLASH_EraseSector(OperationADDR);
  printf("[%s:%d]Erase flash over\n",__func__,__LINE__);

  /* Read from the flash, to check if the flash erased success or not */
  FLASH_Read(pu32Buf, OperationADDR, OperationLEN);
  for(i = 0; i < OperationLEN; i++)
  {
    if(pu32Buf[i] == 0xFFFFFFFF)
    {
      ;
    }
    else
    {
      printf("[%s:%d]Erase flash fail\n",__func__,__LINE__);
      goto WriteData;
    }
  }
  printf("[%s:%d]Erase flash success\n",__func__,__LINE__);

  WriteData:
  /* Generate the data need to be write to flash */
  for(i = 0; i < OperationLEN; i++)
  {
      pu32Buf[i] = (i % 55);
  }
  /* Write data to flash */
  status = FLASH_Program(pu32Buf, OperationADDR, OperationLEN);
  if(status == ERROR)
  {
    printf("[%s:%d]Write to flash ERROR\n \n",__func__,__LINE__);
  }
  else
  {
    printf("[%s:%d]Write flash success\n \n",__func__,__LINE__);
  }
}


/*************************************************************************************************************************
 *
 * @brief      In this case, the partition 0x10000~0x10400 of flash had been set as the protected part, as a result, any 
 *             write and erase to this part will be fail. we need to put the object file flash.o(include all flash operation
 *             program code) into ram, and the other part code into flash, so that, the sys can still get the program data
 *             about flash operation from ram when flash is under processed (XIP was stoped).
 *
 *************************************************************************************************************************/


int main()
{
  FLASH_WALLOW();
  FLASH_SetTiming(200000000);

  CLOCK_InitWithRCO(CLOCK_HCLK_200MHZ);

  Delay_Init();

  /*
   * Init the UART
   *
   * 1.Set the GPIO34/35 as UART FUNC
   *
   * 2.Enable the UART CLK
   *
   * 3.Set the rest para
   */
  GPIO_SetPinChannel(GPIO_34,GPIO34_UART_TXD);
  GPIO_SetPinChannel(GPIO_35,GPIO35_UART_RXD);
  CLOCK_EnableModule(UART_MODULE);
  UART_Init(UART,38400);
  
  /* Write some datum into the sector be going to be protected */
  for(i = 0; i < OperationLEN; i++)
  {
      pu32Buf[i] = (i % 55);
  }
  /* Write data to flash */
  status = FLASH_Program(pu32Buf, OperationADDR, OperationLEN);
  if(status == ERROR)
  {
    printf("[%s:%d]Write to flash ERROR\n",__func__,__LINE__);
    return 0;
  }
  else
  {
    printf("[%s:%d]Write flash success\n",__func__,__LINE__);
  }

  /* Set 0x10000~0x10400 as the write protected area */
  FLASH_WriteProtect(WP_START_ADDR, WP_END_ADDR);

  printf("Flash is working at 200MHz, and Start to set some sectors into protected model...\n");
  Flash_Operation();

  /* Disable flash write access after flash operation had done */
  FLASH_WDIS();

  while(1)
  {

  }
}
