/******************************************************************************
 * @file     main.c
 * @brief    Main program body
 * @version  V1.8.0
 * @date     31-Dec-2021
 *
 * @note
 * Copyright (C) 2021 Spintrol Limited Corporation. All rights reserved.
 *
 * @attention
 * THIS SOFTWARE JUST PROVIDES CUSTOMERS WITH CODING INFORMATION REGARDING
 * THEIR PRODUCTS, WHICH AIMS AT SAVING TIME FOR THEM. SPINTROL SHALL NOT BE
 * LIABLE FOR THE USE OF THE SOFTWARE. SPINTROL DOES NOT GUARANTEE THE
 * CORRECTNESS OF THIS SOFTWARE AND RESERVES THE RIGHT TO MODIFY THE SOFTWARE
 * WITHOUT NOTIFICATION.
 *
 ******************************************************************************/

#include"spc1168.h"
#include<stdio.h>

static int GPIO_level = -1;

int main()
{
  FLASH_WALLOW();
  FLASH_SetTiming(200000000);
  /* Disable flash write access after flash operation had done */
  FLASH_WDIS();

  CLOCK_InitWithRCO(CLOCK_HCLK_200MHZ);

  Delay_Init();

  /*
   * Init the UART
   *
   * 1.Set the GPIO34/35 as UART FUNC
   *
   * 2.Enable the UART CLK
   *
   * 3.Set the rest para
   */
  GPIO_SetPinChannel(GPIO_34,GPIO34_UART_TXD);
  GPIO_SetPinChannel(GPIO_35,GPIO35_UART_RXD);
  CLOCK_EnableModule(UART_MODULE);
  UART_Init(UART,38400);

  /* Set GPIO0 as GPIO FUNC */
  GPIO_SetPinAsGPIO(GPIO_0);

  /* Set GPIO0 direction */
  GPIO_SetPinDir(GPIO_0,GPIO_OUTPUT);

  /* Set the GPIO as low level */
  GPIO_WritePin(GPIO_0,GPIO_LEVEL_LOW);

  /* Toggle the GPIO level */
  GPIO_TogglePin(GPIO_0);

  /* Checking if these configure work */
  GPIO_level = GPIO_ReadPinBitBand(GPIO_0);
  if(GPIO_level != GPIO_LEVEL_LOW)
  {
    printf("GPIO toggle PASS\n");
  }
  else
  {
    printf("GPIO toggle FAIL\n");
  }

  while(1)
  {

  }
}
